/*
 * LuaScriptInvoker.java
 *
 * Created on 25. Mrz 2004, 09:47
 */

package de.hr.xred.hronline.scriptinvoker;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import luajava.*;
/**
 * LuaScriptInvoker invokes LuaScripts
 * @author  TPRAXL
 */
public class LuaScriptInvoker extends ScriptInvoker {
  /**
   *the state of Lua. This is an instance of Lua-Script-Scope
   **/
  protected LuaState l = null;
  String lastError = null;
  /** Creates a new instance of LuaScriptInvoker */
  public LuaScriptInvoker() {
    this.language = "lua";
    this.fileExtension = ".lua";
  }
  /**
   *  initializes the Lua-Environment by creating a state and 
   *  exporting several functions to Lua. This function has to be called
   *  after constructing the invoker.
   *  NOTE: It is essential to call destroy() when destroying the invoker, 
   *  otherwise memory leaks are guaranteed.
   */
  public boolean init(){
    System.out.println("Lua preparing for init");
     if(this.l!=null){
       destroy();
     }
     this.l = LuaStateFactory.newLuaState();
     l.pushJavaFunction(new ObjectCreator(l));
     //l.pushJavaFunction(new URLInclude(l));
     //l.pushJavaFunction(new LuaErrorhandler(l));
     System.out.println("init Lua "+l);
     return true;
  }
  
  /**
   *  executes a script.
   *@param name the name of the script to load and execute
   *@param parameters the parameters for the script. These will be registered as global variables.
   *@return null. Well this is not needed for Lua. So it will always be null.
   *@throws ScriptInvokeException when loading or the execution of the script fails.
   */
  public Object invokeScript(String name, HashMap parameters) throws ScriptInvokeException {
    int idxLastSlash = name.lastIndexOf("/");
    String localPath = "";
    if(idxLastSlash!=-1){
      localPath = name.substring(0,idxLastSlash+1);
    }
    int top = l.getTop();
    l.pushString("LOCAL_PATH"); //export the global variable LOCAL_PATH to the lua environment.
                                //this is needed for the lua-scripts to know their path.
    l.pushString(localPath);
    l.setGlobal();
    if(l.doFile(name)==0){
      if(parameters!=null){
        Set set = parameters.keySet();
        Iterator iter = set.iterator();
        while (iter.hasNext()){
           String key = (String)iter.next();
           l.pushString(key);
           l.pushString((String)parameters.get(key));
           l.setGlobal();
        }
      }
      l.setTop(top);
    }
    else{
      System.out.println("Lua Fehler beim Ausfhren der Datei: "+name);
      l.setTop(top);
      throw (new ScriptInvokeException());
    }
    return null;
  }
  
  /**
   *  executes a function in the given Script
   *@param scriptName the path and name of the script-file to invoke.
   *@param functionName the name of the function to call (without brackets)
   *@param paramters A Hashmap containing the parameterNames as keys and Values as Values.
   *@return the result of the called function as Object
   *@throws ScriptInvokeException when the loaded and executed script produces an error
   *@throws ScriptFunctionException when the called function produces an error
   */
  public Object callScriptFunction(String scriptName, String functionName, HashMap parameters)throws ScriptFunctionException, ScriptInvokeException{
    System.out.println("Lua invoking script "+scriptName+"\nfunction: "+functionName);
    invokeScript(scriptName, null);
    return callScriptFunction(functionName, parameters);
  }
 
  /**
   * executes a function of a loaded script. You want to call invokeScript first.
   *@param functionName the name of the function to call (without brackets)
   *@param parameters A Hashmap containing the parameterNames as keys and Values as Values
   *@return the result of the called function as Object
   *@throws ScriptFunctionException if the function produces an error.
   */
  public Object callScriptFunction(String functionName, HashMap parameters)throws ScriptFunctionException{
    int top = l.getTop();
    if(parameters!=null){
      Set set = parameters.keySet();
      Iterator iter = set.iterator();
      while (iter.hasNext()){
         String key = (String)iter.next();
         l.pushString(key);
         Object value = parameters.get(key);
         if(value instanceof String){
          l.pushString((String)parameters.get(key));
         }
         else{
           l.pushJavaObject(parameters.get(key));
         }
         l.setGlobal();
      }
    }
    l.getGlobal(functionName);
    Object result = null;
    int pcallresult = l.pcall(0,1,0);
    if(pcallresult!=0){
      System.out.println("Lua Fehler: "+l.error());
      result = null;
      l.setTop(top);
      throw (new ScriptFunctionException());
    }
    else{
      result = l.toJavaObject(-1);
      l.setTop(top);
    }
    return result;
  }
  /**
   *  Destroys the Lua-Environment and frees memory.
   *  It is important to explicitely call this method, otherwise, the 
   *  memory will not be deallocated.
   */
  public boolean destroy(){
    
    if(l!=null){
      System.out.println("Lua destroying");
      l.setTop(0);
      l.remove();
      l.close();
      //LuaStateFactory.removeLuaState(0);
      this.l = null;
      return true;
    }
    return false;
    }
  
  public Object doString(String code) throws ScriptFunctionException{
    return doString(code, null);
  }
  
  public Object doString(String code, HashMap parameters) throws ScriptFunctionException{
    int origtop = l.getTop();
    if(parameters!=null){
      Set set = parameters.keySet();
      Iterator iter = set.iterator();
      while (iter.hasNext()){
         String key = (String)iter.next();
         l.pushString(key);
         Object value = parameters.get(key);
         if(value instanceof String){
          l.pushString((String)parameters.get(key));
         }
         else{
           l.pushJavaObject(parameters.get(key));
         }
         l.setGlobal();
      }
    }
    int top = l.getTop();
    int execresult = l.doString(code);
    if(execresult!=0){
      String message = (String)l.toJavaObject(-1);
      System.out.println("Lua Fehler: "+message);
      ScriptFunctionException ex = new ScriptFunctionException("Lua Fehler: "+message);
      throw (ex);
    }
    Object result = null;
    if(top<l.getTop()){
      result = l.toJavaObject(-1);
    }
    l.setTop(origtop);
    return result;
  }
  
}
